# coding=utf-8
from __future__ import unicode_literals
import codecs
import fnmatch
import os

__author__ = 'flanker'


def render_directory_html(self, node, directory, root, options):
    filenames = os.listdir(directory)
    cls = ''
    if node['options'].get('class'):
        cls = ' class="%s"' % node['options']['class']
    glob_txt = node['options'].get('glob')
    regexp = node['options'].get('regexp')
    desc_ext = node['options'].get('desc_ext')
    content = "<ul %s>" % cls
    for filename in filenames:
        if glob_txt and not fnmatch.fnmatch(filename, glob_txt):
            continue
        if regexp and not regexp.match(filename):
            continue
        fullpath = os.path.join(directory, filename)
        if not os.path.isfile(fullpath):
            continue
        if filename.startswith('.'):
            continue
        relpath = os.path.relpath(fullpath, root)
        ext = os.path.splitext(relpath)[1]
        if ext == desc_ext:
            continue
        link = "<a href=\"%s\">%s</a>" % (relpath, filename)
        if desc_ext:
            desc_path = fullpath + desc_ext
            if os.path.isfile(desc_path):
                with codecs.open(desc_path, 'r', encoding='utf-8') as desc_fd:
                    link += "<span>%s</span>" % desc_fd.read()
        content += "<li class=\"%s\">%s</li>\n" % (ext, link)
    content += "</ul>"

    self.body.append(content)


def visit_folder_node(self, node):
    render_directory_html(self, node, node['directory'], node['root'], node['options'])


def depart_folder_node(self, node):
    return



# coding=utf-8
from __future__ import unicode_literals
import re
import os
from sphinxfolders.nodes import FolderNode
from docutils.parsers.rst.directives import unchanged_required
from sphinx.util.compat import Directive

__author__ = 'flanker'


def regexp_check(argument):
    try:
        return re.compile(argument)
    except:
        raise ValueError


def extension_check(argument):
    if not re.match('\.[a-z0-9A-Z]+', argument):
        raise ValueError
    return argument


class FolderDirective(Directive):
    """
    Directive to insert arbitrary dot markup.
    """
    has_content = True
    required_arguments = 1
    final_argument_whitespace = False
    option_spec = {'class': unchanged_required,
                   'empty': unchanged_required,
                   'glob': unchanged_required,
                   'desc_ext': extension_check,
                   'regexp': regexp_check}

    def run(self):
        document = self.state.document
        env = self.state.document.settings.env
        rel_filename, dirname = env.relfn2path(self.arguments[0])
        rel_rootname, root = env.relfn2path('.')
        env.note_dependency(rel_filename)
        try:
            os.listdir(dirname)
        except (IOError, OSError):
            return [document.reporter.warning(
                'Unable to open directory %r' % dirname, line=self.lineno)]
        node = FolderNode()
        node['directory'] = dirname
        node['root'] = root
        node['options'] = self.options
        return [node]



# coding=utf-8
from __future__ import unicode_literals

import codecs
import os.path
import re

from setuptools import setup, find_packages


# avoid a from penatesimport __version__ as version
# (that compiles penates.__init__ and is not compatible with bdist_deb)
version = None
for line in codecs.open(os.path.join('sphinxfolders', '__init__.py'), 'r', encoding='utf-8'):
    matcher = re.match(r"""^__version__\s*=\s*['"](.*)['"]\s*$""", line)
    version = version or matcher and matcher.group(1)

# get README content from README.md file
with codecs.open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as fd:
    long_description = fd.read()

setup(
    name='sphinxfolders',
    version=version,
    description='Sphinx directive to list the content of a folder',
    long_description=long_description,
    author='Matthieu Gallet',
    author_email='github@19pouces.net',
    license='CeCILL-B',
    url='https://github.com/d9pouces/SphinxFolders/',
    entry_points={},
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    test_suite='penates.tests',
    install_requires=['sphinx', 'docutils'],
    setup_requires=[],
    classifiers=[],
)



# coding=utf-8
"""Register all components of this extension"""

from nodes import FolderNode
from sphinxfolders.directives import FolderDirective
from visitors import visit_folder_node, depart_folder_node

__version__ = '1.0.0'


def setup(app):
    # app.add_config_value('todo_include_todos', False, False)

    # app.add_node(todolist)
    app.add_node(FolderNode, html=(visit_folder_node, depart_folder_node))
    app.add_directive('folder', FolderDirective)

#    app.add_directive('todolist', TodolistDirective)
#    app.connect('doctree-resolved', process_folder_nodes)
#    app.connect('env-purge-doc', purge_folders)
